@@ -1,5 +1,7 @@
 import requests
 from .response import Response
+from .emails import Email
+from typing import Union
 
 class Client:
     """ 
@@ -21,7 +23,7 @@ class Client:
         self.apikey = apikey
         self.headers = headers
 
-    def get_headers(self) -> dict:
+    def __get_headers(self) -> dict:
         default_headers = {
             'X-Server-API-Key': self.apikey,
             'content-type': 'application/json'
@@ -31,15 +33,33 @@ class Client:
     def __get_postal_uri(self, resource: str, action: str) -> str:
         return "%s/api/v1/%s/%s" % (self.host, resource, action)
 
-    def __perform_request(
-        self, resource: str, action: str, payload: dict
-    ) -> Response:
+    def __do_request(self, resource: str, action: str, payload: dict):
         return requests.post(
             self.__get_postal_uri(resource, action),
             json = payload,
-            headers = self.get_headers()
+            headers = self.__get_headers()
         )
         
-    async def send(self, payload: dict):
-        response = self.__perform_request('send', 'message', payload)
-        return Response(response)
\ No newline at end of file
+    async def send(self, email: Union[Email,dict]) -> Response:
+
+        # If we receive a simple dict, then we create an Email instance.
+        # This way we are sure the data is set correctly or an Exception
+        # will raise.
+        email = email if isinstance(email, Email) else Email(**email)  
+
+        request = self.__do_request('send', 'message', email.as_payload())
+        response = Response(**request.json())
+        email.set_response(response)
+
+        return response
+
+    async def get_message_details(
+        self, id: int, expansions: list = ['status','details']) -> Response:
+        request = self.__do_request(
+            'messages', 'message', {'id': id, '_expansions': expansions}
+        )
+        return Response(**request.json())
+    
+    async def get_message_deliveries(self, id: int) -> Response:
+        request = self.__do_request('messages', 'deliveries', {'id': id})
+        return Response(**request.json())
\ No newline at end of file
